# -*- coding: utf8 -*-

# nmrtabelle – Python script for the generation of NMR measurement metadata tables
# Copyright (C) 2023  B. Sc. Felix Krickl

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Programm	:	nmrtabelle
# Version   :   1.0
# lang      :   de
# Autor     :	Felix Krickl

import csv
import os
import sys

wd = os.getcwd()
pathToTitleFile = "pdata" + os.sep + "1" + os.sep + "title"
csvFile = wd + os.sep + "nmrtabelle.csv"
noTitle = "no title"
inputNmrData = []
hdl = ["TITLE", "EXPNO", "DATE", "FREQ", "NUCLEUS", "SOLVENT", "NS"]


if __name__ == '__main__':

    print("Python " + sys.version[:7] + ' interpretiert "nmrtabelle" von Felix Krickl …')
    print("Lese Verzeichnisinhalt …")
    allDirs = [d for d in os.listdir() if os.path.isdir(d)]
    nmrDirs = [d for d in allDirs if d.isdigit()]
    otherDirs = [d for d in allDirs if d.isdigit() is False]

    if len(otherDirs) != 0:
        print("WARNUNG: Die folgenden Unterordner werden nicht als NMR-Messordner erkannt:\n")
        print(otherDirs)
        print("Wenn diese Ordner NMR-Messdaten enthalten müssen sie so umbenannt werden, " +
              "dass nur Nummern im Ordnernamen vorkommen.")
    if len(nmrDirs) == 0:
        sys.exit("FEHLER: Keine NMR-Messordner gefunden. Nur Ordner mit Nummern als Name werden beachtet.\n\
Kopieren Sie Messordner in dieses Verzeichnis oder löschen Sie alle Zeichen in den Ordnernamen, \
die keine Nummern sind.")
    else:
        print("Folgende Unterordner werden als Messordner interpretiert:")
        print(nmrDirs)

    for nmrDir in nmrDirs:

        titleFile = wd + os.sep + nmrDir + os.sep + pathToTitleFile
        acquisitionFile = wd + os.sep + nmrDir + os.sep + "acqu"
        curDataLine = []

        if os.path.isfile(titleFile):

            try:
                with open(titleFile, 'r') as f:
                    curTitle = f.read().replace('\n', ', ').strip()[:-1]
            except UnicodeError:
                print("WARNUNG: In der Titel-Datei des Messordners "
                      + nmrDir + " wurde ein ungültiges Zeichen gefunden.")
                print("Öffnen Sie die Datei " + titleFile + "  und löschen Sie ungültige Zeichen.")
                curTitle = ""
            if len(curTitle) == 0 or curTitle.isspace():
                curDataLine.append(noTitle)
                curDataLine.append(nmrDir)
            else:
                curDataLine.append(curTitle)
                curDataLine.append(nmrDir)

        else:
            print("WARNUNG: Keine Titel-Datei gefunden für Messordner " + nmrDir)
            print("Sie können die Datei " + titleFile + " manuell anlegen und dort den Messtitel einfügen.")
            curDataLine.append(noTitle)
            curDataLine.append(nmrDir)

        if os.path.isfile(acquisitionFile):
            with open(acquisitionFile, 'r') as aF:
                lines = aF.readlines()
                for line in lines:
                    lineNumber = lines.index(line)
                    if line[:2].__eq__("$$") and line[3:7].isdigit():
                        curDate = line[3:13]
                    if line[:7].__eq__("##$SFO1"):
                        curFreq = line[9:12]
                    if line[:7].__eq__("##$NUC1"):
                        curNuc = line.split(" ")[1][1:-2]
                    if line[:10].__eq__("##$SOLVENT"):
                        curSolvent = line.split(" ")[1][1:-2]
                    if line[:5].__eq__("##$NS"):
                        curNS = line.split(" ")[1][:-1]
                if not len(curDate) == 0:
                    curDataLine.append(curDate)
                else:
                    curDataLine.append("-")
                if not len(curFreq) == 0:
                    curDataLine.append(curFreq)
                else:
                    curDataLine.append("-")
                if not len(curNuc) == 0:
                    curDataLine.append(curNuc)
                else:
                    curDataLine.append("-")
                if not len(curSolvent) == 0:
                    curDataLine.append(curSolvent)
                else:
                    curDataLine.append("-")
                if not len(curNS) == 0:
                    curDataLine.append(curNS)
                else:
                    curDataLine.append("-")
        else:
            print("WARNUNG: Keine acquisition Datei gefunden für Messordner " + nmrDir)
            print("Suchen Sie auf dem NMR Server nach der fehlenden Datei.")

        inputNmrData.append(curDataLine)

    if os.path.isfile(csvFile):
        print("WARNUNG: Es existiert bereits eine Tabellen-Datei.")
        print('Die bestehende Tabellen-Datei wird nach "nmrtabelle_old.csv" verschoben …')
        os.rename(csvFile, 'nmrtabelle_old.csv')
    with open(csvFile, 'w') as oF:
        writer = csv.writer(oF)
        writer.writerow(hdl)
        for row in inputNmrData:
            writer.writerow(row)

    sys.exit('"nmrtabelle" ohne Fehler beendet.')
